package com.faraphel.tasks_valider.connectivity.task

import com.faraphel.tasks_valider.connectivity.task.api.TaskSessionManagerClientApi
import com.faraphel.tasks_valider.database.api.client.TaskEntityHttpClient
import com.faraphel.tasks_valider.database.api.client.entities.*


/**
 * A client to handle the room connection and access the API
 * @param address the address of the server
 * @param port the port of the server
 * @param baseCookies list of cookies to use (optional)
 */
class TaskClient(
    private val address: String,
    private val port: Int,
    private val baseCookies: List<okhttp3.Cookie> = listOf()
) {
    private val httpClient = TaskEntityHttpClient(address, port, baseCookies)

    // all the entities API
    class Entities(httpClient: TaskEntityHttpClient) {
        val client = ClassClientApi(httpClient)
        val person = PersonClientApi(httpClient)
        val session = SessionClientApi(httpClient)
        val subject = SubjectClientApi(httpClient)
        val task = TaskClientApi(httpClient)
        val validation = ValidationClientApi(httpClient)

        val relationClassPerson = RelationClassPersonClientApi(httpClient)
        val relationPersonSessionSubject = RelationPersonSessionSubjectClientApi(httpClient)
    }
    val entities = Entities(httpClient)

    val session = TaskSessionManagerClientApi(httpClient)
}
