package com.faraphel.tasks_valider.database.dao

import androidx.room.Dao
import androidx.room.Query
import com.faraphel.tasks_valider.database.dao.base.BaseCronDao
import com.faraphel.tasks_valider.database.dao.base.BaseSessionDao
import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.RelationPersonSessionSubjectEntity
import com.faraphel.tasks_valider.database.entities.TaskEntity
import com.faraphel.tasks_valider.database.entities.ValidationEntity

@Dao
interface TaskDao : BaseTaskDao<TaskEntity> {
    @Query("SELECT * FROM ${TaskEntity.TABLE_NAME}")
    override fun getAll(): List<TaskEntity>

    @Query(
        "SELECT * FROM ${TaskEntity.TABLE_NAME} " +
        "WHERE subject_id IN (" +
            "SELECT subject_id FROM ${RelationPersonSessionSubjectEntity.TABLE_NAME} " +
            "WHERE session_id = :sessionId" +
        ")"
    )
    override fun getAllBySession(sessionId: Long): List<TaskEntity>

    /**
     * Get the object from its identifier
     */
    @Query("SELECT * FROM ${TaskEntity.TABLE_NAME} WHERE id = :id")
    fun getById(id: Long): TaskEntity?

    /**
     * Get all the validations have been approved for this tasks
     * @param id the id of the task
     */
    @Query("SELECT * FROM ${ValidationEntity.TABLE_NAME} WHERE task_id = :id")
    fun getTasksValidated(id: Long): List<ValidationEntity>
}
