package com.faraphel.tasks_valider.database.entities

import androidx.room.ColumnInfo
import androidx.room.Entity
import androidx.room.ForeignKey
import com.faraphel.tasks_valider.database.entities.base.BaseEntity
import com.google.gson.annotations.Expose
import java.time.Instant

@Entity(
    tableName = ValidationEntity.TABLE_NAME,
    primaryKeys = [
        "teacher_id",
        "student_id",
        "task_id",
    ],
    foreignKeys = [
        ForeignKey(
            entity = PersonEntity::class,
            parentColumns = ["id"],
            childColumns = ["teacher_id"],
            onDelete = ForeignKey.CASCADE
        ),
        ForeignKey(
            entity = PersonEntity::class,
            parentColumns = ["id"],
            childColumns = ["student_id"],
            onDelete = ForeignKey.CASCADE
        ),
        ForeignKey(
            entity = TaskEntity::class,
            parentColumns = ["id"],
            childColumns = ["task_id"],
            onDelete = ForeignKey.CASCADE
        ),
    ]
)
data class ValidationEntity (
    @ColumnInfo("teacher_id", index = true) @Expose val teacherId: Long,
    @ColumnInfo("student_id", index = true) @Expose val studentId: Long,
    @ColumnInfo("task_id", index = true) @Expose val taskId: Long,

    @ColumnInfo("date") @Expose val date: Instant,
) : BaseEntity() {
    companion object {
        const val TABLE_NAME = "validations"
    }

    /**
     * Construct a new ValidationEntity. Automatically set the date to today.
     */
    constructor(
        teacherId: Long,
        studentId: Long,
        taskId: Long
    ):
        this(
            teacherId = teacherId,
            studentId = studentId,
            taskId = taskId,
            date = Instant.now()
        )
}
