package com.faraphel.tasks_valider.ui.screen.communication.connection.wifiP2p.role

import android.app.Activity
import android.net.wifi.p2p.WifiP2pConfig
import android.net.wifi.p2p.WifiP2pDevice
import androidx.compose.foundation.layout.Column
import androidx.compose.runtime.*
import com.faraphel.tasks_valider.connectivity.bwd.BwdManager
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.session.TaskSession
import com.faraphel.tasks_valider.ui.screen.communication.DEFAULT_SERVER_PORT
import com.faraphel.tasks_valider.ui.screen.communication.authentication.AuthenticationClientScreen
import com.faraphel.tasks_valider.ui.screen.communication.connection.internet.role.CommunicationInternetClientContent
import com.faraphel.tasks_valider.ui.screen.task.TaskSessionController
import com.faraphel.tasks_valider.ui.screen.task.TaskSessionScreen
import com.faraphel.tasks_valider.ui.widgets.connectivity.WifiP2pDeviceListWidget


@Composable
fun CommunicationWifiP2pClientScreen(activity: Activity, bwdManager: BwdManager) {
    val client = remember { mutableStateOf<TaskClient?>(null) }
    val session = remember { mutableStateOf<TaskSession?>(null) }

    // if the client is not connected, show the connection screen
    if (client.value == null)
        return CommunicationWifiP2pClientContent(client, bwdManager)
    // if the user is not authenticated, show the authentication screen
    if (session.value == null)
        return AuthenticationClientScreen(activity, client.value!!, session)

    // show the main screen
    TaskSessionController(
        activity,
        client.value!!,
        session.value!!.person,
    )
}


@Composable
fun CommunicationWifiP2pClientContent(
    client: MutableState<TaskClient?>,
    bwdManager: BwdManager,
) {
    val selectedDevice = remember { mutableStateOf<WifiP2pDevice?>(null) }

    if (selectedDevice.value == null) {
        Column {
            WifiP2pDeviceListWidget(
                peers = bwdManager.statePeers.value,
                filter = { device: WifiP2pDevice -> device.isGroupOwner },
                selectedDevice,
            )
        }

        LaunchedEffect(true) {
            // look for new peers
            bwdManager.discoverPeers()
        }

        return
    }

    val config = WifiP2pConfig().apply {
        deviceAddress = selectedDevice.value!!.deviceAddress
    }
    bwdManager.connect(config) {
        bwdManager.requestConnectionInfo { connectionInfo ->
            // TODO(Faraphel): check if the server is reachable
            client.value = TaskClient(
                connectionInfo.groupOwnerAddress.toString(),
                DEFAULT_SERVER_PORT
            )
        }
    }
}