package com.faraphel.tasks_valider.ui.screen.communication.connection.wifiP2p.role

import android.app.Activity
import android.util.Log
import androidx.compose.foundation.layout.*
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.bwd.BwdManager
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.TaskServer
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.entities.ClassEntity
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.faraphel.tasks_valider.database.entities.SessionEntity
import com.faraphel.tasks_valider.database.populateSubjectSessionPersonTest
import com.faraphel.tasks_valider.ui.screen.communication.DEFAULT_SERVER_PORT
import com.faraphel.tasks_valider.ui.screen.communication.authentication.AuthenticationServerScreen
import com.faraphel.tasks_valider.ui.screen.task.TaskSessionController
import java.time.Instant


@Composable
fun CommunicationWifiP2pServerScreen(
    activity: Activity,
    database: TaskDatabase,
    bwdManager: BwdManager
) {
    val controller = rememberNavController()

    val adminPersonEntityRaw = remember { mutableStateOf<PersonEntity?>(null) }
    val adminPersonEntity = remember { mutableStateOf<PersonEntity?>(null) }
    val client = remember { mutableStateOf<TaskClient?>(null) }

    NavHost(navController = controller, startDestination = "authentication") {
        composable("authentication") {
            // if the admin person is not created, prompt the user for the admin information
            if (adminPersonEntityRaw.value == null) AuthenticationServerScreen(adminPersonEntityRaw)
            else controller.navigate("configuration")
        }
        composable("configuration") {
            if (client.value == null)
                CommunicationWifiP2pServerContent(
                    database,
                    bwdManager,
                    adminPersonEntityRaw,
                    adminPersonEntity,
                    client,
                )
            else controller.navigate("session")
        }
        composable("session") {
            TaskSessionController(
                activity,
                client.value!!,
                adminPersonEntity.value!!
            )
        }
    }
    // TODO(Faraphel): fix and get a user
    // if the server is not created, prompt the user for the server configuration
    // if (client.value == null) CommunicationWifiP2pServerContent(activity, bwfManager, client)
    // else, go to the base tasks screen
    // else TaskSessionScreen(activity, client.value!!)
}


@Composable
fun CommunicationWifiP2pServerContent(
    database: TaskDatabase,
    bwdManager: BwdManager,
    adminPersonEntityRaw: MutableState<PersonEntity?>,
    adminPersonEntity: MutableState<PersonEntity?>,
    client: MutableState<TaskClient?>
) {
    val classes = remember { mutableStateOf<List<ClassEntity>?>(null) }
    val selectedClass = remember { mutableStateOf<ClassEntity?>(null) }
    val areClassesExpanded = remember { mutableStateOf(false) }

    LaunchedEffect(true) {
        // refresh the list of classes
        Thread { refreshClasses(database, classes) }.start()
    }

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "New Session",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // classes
        Row(verticalAlignment = Alignment.CenterVertically) {
            // description
            Text(text = "Class", fontSize = 12.sp)
            // separator
            Spacer(modifier = Modifier.width(width = 12.dp))
            // selector
            Button(onClick = { areClassesExpanded.value = !areClassesExpanded.value }) {
                // display the selected class, if selected
                if (selectedClass.value != null) Text(text = selectedClass.value!!.name)
                else Text(text = "<Not selected>")
            }

            // class selector
            DropdownMenu(
                expanded = areClassesExpanded.value,
                onDismissRequest = { areClassesExpanded.value = false }
            ) {
                // TODO(Faraphel): student lists should be loaded from the database or a file
                classes.value?.forEach { class_ ->
                    DropdownMenuItem(
                        text = { Text(class_.name) },
                        onClick = {
                            selectedClass.value = class_
                            areClassesExpanded.value = false
                        }
                    )
                }
            }
        }

        // check if a class is selected
        if (selectedClass.value != null)
        // button to create the server
            Button(onClick = {
                Thread {  // a thread is used for networking
                    // Insert the admin in the database and get its id
                    val adminPersonEntityId = database.personDao().insert(adminPersonEntityRaw.value!!)
                    adminPersonEntity.value = database.personDao().getById(adminPersonEntityId)!!

                    // Create a new session
                    // TODO(Faraphel): name
                    val sessionId = database.sessionDao().insert(
                        SessionEntity(
                            name="NOM",
                            start= Instant.now(),
                            classId=selectedClass.value!!.id,
                        )
                    )
                    val session = database.sessionDao().getById(sessionId)!!

                    // TODO(Faraphel): remove, this is a test function
                    populateSubjectSessionPersonTest(database, session)

                    // create a new Wi-Fi Direct group
                    bwdManager.recreateGroup {
                        // Create the server
                        Log.i("room-server", "creating the server")
                        val server = TaskServer(
                            DEFAULT_SERVER_PORT,
                            database,
                            session,
                            adminPersonEntity.value!!,
                        )
                        server.start()

                        // Get the client from the server
                        client.value = server.getAdminClient()
                    }
                }.start()
            }) {
                Text("Create")
            }
    }
}


/**
 * Refresh the list of classes
 */
fun refreshClasses(database: TaskDatabase, classes: MutableState<List<ClassEntity>?>) {
    classes.value = database.classDao().getAll()
}
