package com.faraphel.tasks_valider.utils.converters

import androidx.room.TypeConverter
import com.google.gson.*
import java.lang.reflect.Type
import java.time.Instant


/**
 * Allow for automatically converting Instant values when using them in the database or in a json parser.
 */
class InstantConverter : JsonDeserializer<Instant>, JsonSerializer<Instant> {
    /**
     * Convert a long into an instant
     * @param value the number of milliseconds since the epoch of the time
     * @return the Instant object
     */
    @TypeConverter
    fun deserialize(value: Long): Instant {
        return Instant.ofEpochMilli(value)
    }

    /**
     * Convert a long into an instant
     * @param instant the Instant object
     * @return the number of milliseconds since the epoch of the time
     */
    @TypeConverter
    fun serialize(instant: Instant): Long {
        return instant.toEpochMilli()
    }

    /**
     * Convert a long into an instant
     * @param json the json object
     * @return the Instant object
     */
    override fun deserialize(json: JsonElement, type: Type, context: JsonDeserializationContext): Instant =
         this.deserialize(json.asLong)

    /**
     * Convert a long into an instant
     * @param instant the instant object
     * @return the json object
     */
    override fun serialize(instant: Instant, type: Type, context: JsonSerializationContext): JsonElement =
        JsonPrimitive(this.serialize(instant))
}
